<?php
/**
 * Projekt: WS-Reporter
 *          Funktionen für mehrsprachige Datenquellen und Reports
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   GPL 3.0
 *
 * Stand:    16.6.2022
 *
 * FEHLER:
 *  1. translateTable funktioniert nicht, weil die Texte UNIQUE sind und 
 *     ein kombinierter Index mit qLanguageType zu lang wäre.
 *     Beispiel Ergebnisse wird als Text übersetzt, der Tabellenname soll 
 *     aber nicht übersetzt werden.
 *  2. Es gibt noch keine Lösung für Report-Namen, weil der Report-Title
 *     gleichzeitig das Schlüsselfeld der Report-Definition ist.
 *     Überlegungen:
 *     Report-Tiitle kann nicht in der Sprache rLanguage gespeichert werden,
 *     weil die vor dem Einlesen des ersten Records nicht bekannt ist.
 *     Also muss die Speicherung in de erfolgen.
 *     In Fremdsprachen sind dann aber nur Reports mit übersetzten Title,
 *     d.h. eingetragenen Übersetzungen verfügbar.
 *     Alternative: Reverse Translation: Es wird aus $_SESSION['Sprache']
 *     oder $query->currentLanguage (???) nach de übersetzt, um den Schlüssel
 *     zu ermitteln. 
 *     Beim Report-Title kommt nur $_SESSION['Sprache'] in Frage, weil 
 *     in vielen Fällen currentLanguage nicht bekannt ist. 
 *     Zusätzlich muss ein "statischer" Aufruf mit deutschem Title 
 *     ohne Übersetzung implementiert werden.
 *     Was ist mit dem Speichern?
 *     a) Ganz neuer Report anlegen:
 *     b) Report ohne Namensänderung speichern:
 *     c) Report mit geändertem Namen (Übersetzung) speichern:
 *     d) Geänderter Report unter neuem Namen (zuätzlich) speichern:           
 *     e) Geänderter Report unter neuem Namen ersetzt anderen Report (überschreiben):
 *            
 *     Saubere Lösung erfordert neues Konzept mit sprachunabhängigem Schlüssel!!!
 *     Dabei ist BIGINT unproblematisch! Großer Wertebereich.
 *     reportID SERIAL
 *     insert ohne reportID
 *     SELECT LAST_INSERT_ID() liefert die reportID
 *     
 *     Ist stabiler Schlüssel für statische Aufrufe - siehe unten: Beim Ersetzen bleibt die reportID erhalten
 *     Report-Definitionen mit inhaltlichen Spaltennamen neu gestalten?
 *     Dann ggf. auch verschiedene Tabellen??
 *     Neue Funktionen:
 *     * neuer Report: Legt sofort neuen Eintrag an. Timestamp zum Aufräumen wenn Title leer bleibt?
 *                     Abbrechen oder wechseln ohne vorheriges Speichern löscht den alten Eintrag
 *                     $query->reportIsStored
 *                     $query->reportID 
 *     * Report lesen: $query->reportIsStored = true 
 *     * Report speichern: prüft Eindeutigkeit des Titles
 *     * Report speichern unter:
 *                     Ersetzen oder neuen Eintrag anlegen, prüft Eindeutigkeit des Titles   
 *     * Report löschen:
 *     
 *     Funktionen in Klasse Query, damit reportIsStored und reportID gepflegt werden.
 *     Dort kann auch die Sprache "behandelt" werden                                   
 */

require_once './phpsec/createDBH.inc';
require_once './includes/funktionen.inc';
require_once './includes/classes.inc';

/**
 * Konstanten um verschiedene Übersetzungsbereiche zu separieren.
 * @var integer
 */

class qTranslate{
    const QT_LABEL = 1;
    const QT_TEXT = 2;
    const QT_SQLDISP = 3;
    const QT_SQL  = 4;
    const QT_SQLTABLE = 5;
    const QT_WEBLINK = 6;
    const QT_REPORT = 101;

    const QT_QUERY = 100;

    const QUERY_COMMENT = 1002;
    const QUERY_HINTS = 1003;
    
    const REPORT_TITLE = 2001;
    const REPORT_COMMENT = 2002;
    const REPORT_HINTS = 2003;
    
    const SPRACHEN = array('de','en','fr');
    
    static $saveReportTitle = null;
    static $readStatement = null;
    static $readStatement1 = null;
    static $readTableStatement = null;
    
    /**
     * Klartext für Sprachen
     * @param string $s
     * @return string
     */
    static function sprache($s){
        switch (substr($_SESSION['Sprache'],0,2)){
        case 'en':
        if (! isValid($s)) return 'unknown';        
        switch (substr($s, 0,2)){
            case 'de': return 'German';
            case 'en': return 'English';
            case 'fr': return 'French';
            default: return 'invalid: '.$s;
        }
        case 'fr':
            if (! isValid($s)) return 'inconnue';
            switch (substr($s, 0,2)){
                case 'de': return 'Allemand';
                case 'en': return 'Anglais';
                case 'fr': return 'Français';
                default: return 'invalide: '.$s;
            }
        default:
            if (! isValid($s)) return 'unbekannt';
            switch (substr($s, 0,2)){
                case 'de': return 'Deutsch';
                case 'en': return 'Englisch';
                case 'fr': return 'Französisch';
                default: return 'ungültig: '.$s;
            }
        }
    }
    
    /**
     * Texte ohne Bezug zu einer Query übersetzen.
     * @param string $text
     * @return string übersetzter Text
     */
    public static function translate($text){
        global $dbh;
        $text = wsrDecode($text);
        
        $sprache = substr($_SESSION['Sprache'], 0,2);
//        error_log('Translate nach '.$sprache.': '.$text);
        if ($sprache == 'de') return $text;
        if (self::$readStatement1 == null){
            $statement = 'select '.$sprache.' as trans from queryTranslations where de = :text';
            self::$readStatement1 = $dbh->prepare($statement);
            if (self::$readStatement1 === false){
                echo "self Query FEHLER:\n".$dbh->errorInfo()[2]."\n".$statement;
                die();
            }
        }
        if (false === self::$readStatement1->execute (array('text'=> $text ))){
            echo "self Query READ-FEHLER:\n".self::$readStatement1->errorInfo()[2]."\n".$statement;
            echo "\nText=".$text;
            die();
        }
        $z = self::$readStatement1->fetchObject();
        if ($z == false || ! isValid($z->trans)) {
            return $text;
        }        
        return $z->trans;
    }
    
    
    
    
    /**
     * Texte beim Einlesen einer Query übersetzen.
     * @param Query $query
     * @param string $text
     * @return string übersetzter Text
     */
    public static function translateQuery($query,$text){
        global $dbh;
        $text = html_entity_decode($text);
        $text = trim(str_replace("\r", '',$text));
        
        $sprache = $query->currentLanguage;
        $oSprache = $query->qLanguage;
        if ($sprache == $oSprache) return $text;
        if (self::$readStatement == null){
            $statement = 'select '.$sprache.' as trans from queryTranslations where '.$oSprache.' = :text';
            self::$readStatement = $dbh->prepare($statement);
            if (self::$readStatement === false){
                echo "self Query FEHLER:\n".$dbh->errorInfo()[2]."\n".$statement;
                die();
            }
        }
        if (false === self::$readStatement->execute (array('text'=> $text ))){ 
            echo "self Query READ-FEHLER:\n".self::$readStatement->errorInfo()[2]."\n".$statement;
            echo "\nText=".$text;
            die();
        }
        $z = self::$readStatement->fetchObject();
        if ($z == false || ! isValid($z->trans)) {
            return $text;
        }
        return $z->trans;        
    }

    /**
     * Übersetzte Tabelle zu einer Query finden.
     * Das funktioniert so nicht wirklich und steht hier nur, weil es noch gemacht werden muss.
     * @param Query $query
     * @param string $text Tabelle
     * @return string übersetzte Tabelle
     */
    public static function translateTable($query,$text){
        global $dbh;
        $sprache = $query->currentLanguage;
        $oSprache = $query->qLanguage;
        $text = html_entity_decode($text);
        $text = trim(str_replace("\r", '',$text));
        
        if ($sprache == $oSprache) return $text;
        if (self::$readTableStatement == null){
            $statement = 'select '.$sprache.' as trans from queryTranslations where '.$oSprache.' = :text';
            $statement .= ' and qLanguageType = '.self::QT_SQLTABLE;
            self::$readTableStatement = $dbh->prepare($statement);
            if (self::$readTableStatement === false){
                echo "self Query Table FEHLER:\n".$dbh->errorInfo()[2]."\n".$statement;
                die();
            }
        }
        if (false === self::$readTableStatement->execute (array('text'=> $text ))){  
            echo "self Query Table READ-FEHLER:\n".self::$readTableStatement->errorInfo()[2]."\n".$statement;
            echo "\nText=".$text;
            die();
        }
        $z = self::$readTableStatement->fetchObject();
        if ($z == false || ! isValid($z->trans)) return $text;
        return $z->trans;
    }
    
    
    
    /**
     * Texte beim Einlesen eines Reports übersetzen.
     * @param Query $query
     * @param string $text
     * @return string übersetzter Text
     */
    public static function translateReport($query,$text){
        global $dbh;
        $text = html_entity_decode($text);
        $text = trim(str_replace("\r", '',$text));
        
        $sprache = $query->currentRLanguage;
        $oSprache = $query->rLanguage;
//        error_log('Übersetze Report von '.$oSprache.' nach '.$sprache.': '.$text);
        if ($sprache == $oSprache) return $text;
        if (self::$readStatement == null){
            $statement = 'select '.$sprache.' as trans from queryTranslations where '.$oSprache.' = :text'; // and qLanguageType = :type';
            self::$readStatement = $dbh->prepare($statement);
            if (self::$readStatement === false){
                echo "self Report FEHLER:\n".$dbh->errorInfo()[2]."\n".$statement;
                die();
            }
        }
        if (false === self::$readStatement->execute (array('text' => $text))){  // , 'type'=>$type
            echo "self Report READ-FEHLER:\n".self::$readStatement->errorInfo()[2]."\n".$statement;
            echo "\nText=".$text;
            die();
        }
        $z = self::$readStatement->fetchObject();
        if ($z == false || ! isValid($z->trans)) return $text;
//        error_log('- Ergebnis '.$z->trans);
        return $z->trans;
    }

    /**
     * Texte beim Einlesen eines Weblinks übersetzen.
     * Wenn hier ein Report-Title steht, muss über reportTitle übersetzt werden 
     * @param Query $query
     * @param string $text
     * @return string übersetzter Text
     */
    public static function translateWebLink($query,$text){
        global $dbh;
        if (isValid($text) && substr($text, 0,1) == '#') return self::translateReport($query,$text);
        $text = html_entity_decode($text);
        $text = trim(str_replace("\r", '',$text));
        if (isset($_SESSION['Sprache'])) $sprache = substr($_SESSION['Sprache'], 0,2);
        else $sprache = $query->currentRLanguage;
        
        $oSprache = $query->rLanguage;
//        error_log('Übersetze Report-Web-Link von '.$oSprache.' nach '.$sprache.': '.$text);
        if ($sprache == $oSprache) return $text;
        $statement = 'select reportTitle_'.$sprache.' as trans from reportTitle where reportTitle_'.$oSprache.' = :text'; 
        $readStatement = $dbh->prepare($statement);
            if ($readStatement === false){
                echo "Translate Web-Link Report FEHLER:\n".$dbh->errorInfo()[2]."\n".$statement;
                die();
            }
        
        if (false === $readStatement->execute (array('text' => $text))){  // , 'type'=>$type
            echo "Translate Web-Link Report READ-FEHLER:\n".$readStatement->errorInfo()[2]."\n".$statement;
            echo "\nText=".$text;
            die();
        }
        $z = $readStatement->fetchObject();
        if ($z == false || ! isValid($z->trans)) return $text;
//        error_log('- Ergebnis '.$z->trans);
        return $z->trans;
    }
    
    /**
     * Report-Title in Benutzer-Sprache liefern
     * Wenn hier ein Report-Title steht, muss über reportTitle übersetzt werden
     * @param string $text
     * @return string übersetzter Text
     */
    public static function translateReportTitle($text){
        global $dbh;
        $text = html_entity_decode($text);
        $text = trim(str_replace("\r", '',$text));
        
        if (isset($_SESSION['Sprache'])) $sprache = substr($_SESSION['Sprache'], 0,2);
        else return ($text);
//        error_log('Translate title '.$text.' nach '.$sprache);
        if ($sprache == 'de') return $text;
        $statement = 'select reportTitle_'.$sprache.' as trans from reportTitle where reportTitle_de = :text';
        $readStatement = $dbh->prepare($statement);
        if ($readStatement === false){
            echo "self Report FEHLER:\n".$dbh->errorInfo()[2]."\n".$statement;
            die();
        }
        
        if (false === $readStatement->execute (array('text' => $text))){  // , 'type'=>$type
            echo "self Report READ-FEHLER:\n".$readStatement->errorInfo()[2]."\n".$statement;
            echo "\nText=".$text;
            die();
        }
        $z = $readStatement->fetchObject();
        if ($z == false || ! isValid($z->trans)) return $text;
                // error_log('- Ergebnis '.$z->trans);
        return $z->trans;
    }
    
    
    
    
    /**
     * Liest die Übersetzte Beschreibung der Datenquelle ein.
     * @param Query $query
     * @return bool TRUE, wenn übersetzung gefunden wurde. 
     */    
    public static function readQueryDescription($query){
        global $dbh;
        $sprache = $query->currentLanguage;
        $oSprache = $query->qLanguage;
        if ($sprache == $oSprache) return true;
        elseif (array_search($oSprache,$query->qTranslations) === false) return false;
        
        /* noch implementieren */
        return false;
    }

}
